@@ -8,6 +8,8 @@
 *   12/20/04   DGP   Added MAP support and 2.0 additions.
 *   02/03/05   DGP   Added JOBSYM mode.
 *   02/08/05   DGP   Added FAP support and multiple assembly.
+*   03/08/05   DGP   Added CPU model.
+*   03/25/05   DGP   Added 704 model and instructions.
 *	
 ***********************************************************************/
 
@@ -21,6 +23,7 @@
 #include <time.h>
 
 #include "asmdef.h"
+#include "asmdmem.h"
 
 int absolute;			/* In absolute section */
 int absmod;			/* In absolute module */
@@ -52,6 +55,7 @@ int qualindex;			/* QUALifier table index */
 int litorigin;			/* Literal pool origin */
 int litpool;			/* Literal pool size */
 int headcount;			/* Number of entries in headtable */
+int cpumodel;			/* CPU model (709, 7090 7094) */
 
 char errline[10][120];		/* Pass 2 error lines for current statment */
 char inbuf[MAXLINE];		/* The input buffer for the scanners */
@@ -706,6 +710,7 @@ main (int argc, char **argv)
 
    filecnt = 0;
    p1erralloc = 0;
+   cpumodel = 7090;
 
    deckname[0] = '\0';
    titlebuf[0] = '\0';
@@ -720,13 +725,27 @@ main (int argc, char **argv)
 
       if (*bp == '-')
       {
-         bp++;
-         switch (*bp)
+	 for (bp++; *bp; bp++) switch (*bp)
          {
 	 case 'a': /* Absolute assembly */
 	    gblabsmod = TRUE;
 	    break;
 
+	 case 'c': /* CPU model */
+	    i++;
+	    cpumodel = atoi(argv[i]);
+	    switch (cpumodel)
+	    {
+	    case 704:
+	    case 709:
+	    case 7090:
+	    case 7094:
+	       break;
+	    default:
+	       goto USAGE;
+	    }
+	    break;
+
 	 case 'd': /* Deckname */
 	    i++;
 	    if (strlen (argv[i]) > MAXSYMLEN) argv[i][MAXSYMLEN] = '\0';
@@ -787,9 +806,11 @@ main (int argc, char **argv)
       USAGE:
 	    fprintf (stderr, "asm7090 - version %s\n", VERSION);
 	    fprintf (stderr,
-		     "usage: asm7090 [-options] -o file.bin file.asm\n");
+		     "usage: asm7090 [-options] -o file.obj file.asm\n");
             fprintf (stderr, " options:\n");
 	    fprintf (stderr, "    -a           - Absolute assembly\n");
+	    fprintf (stderr,
+		     "    -c model     - CPU model (704, 709, 7090, 7094)\n");
 	    fprintf (stderr, "    -d deck      - Deckname for output\n");
 	    fprintf (stderr, "    -f           - FAP assembly mode\n");
 	    fprintf (stderr, "    -j           - JOBSYM mode\n");
@@ -881,7 +902,7 @@ main (int argc, char **argv)
 #ifdef WIN32
    sprintf (tname0, "%s1.s", bp);
 #else
-   sprintf (tname0, "/tmp/%s.s", bp);
+   sprintf (tname0, "/tmp/%s1.s", bp);
 #endif
 #ifdef DEBUGTMPFILE
    printf ("asm7090: tmp0 = %s\n", tname0);
@@ -903,7 +924,7 @@ main (int argc, char **argv)
 #ifdef WIN32
    sprintf (tname1, "%s2.s", bp);
 #else
-   sprintf (tname1, "/tmp/%s.s", bp);
+   sprintf (tname1, "/tmp/%s2.s", bp);
 #endif
 #ifdef DEBUGTMPFILE
    printf ("asm7090: tmp1 = %s\n", tname1);
@@ -1053,7 +1074,9 @@ main (int argc, char **argv)
 	 if (verbose)
 	    fprintf  (stderr, "asm7090: File %d: EOF\n",
 		     filecnt+1);
-	 fclose (tmpfd1);
+	 if (tmpfd1)
+	    fclose (tmpfd1);
+	 tmpfd1 = NULL;
 	 status = 0;
          done = TRUE;
 	 break;
@@ -1084,6 +1107,8 @@ main (int argc, char **argv)
 	    fprintf (stderr, "asm7090: Calling pass %03o\n", inpass);
 
 	 status = asmpass1 (tmpfd1, FALSE);
+	 status = asmpass1 (tmpfd1, FALSE);
+
 	 inpass = 012;
 	 if (verbose)
 	    fprintf (stderr, "asm7090: Calling pass %03o\n", inpass);
@@ -1105,7 +1130,9 @@ main (int argc, char **argv)
       ** Close the intermediate file
       */
 
-      fclose (tmpfd1);
+      if (tmpfd1)
+	 fclose (tmpfd1);
+      tmpfd1 = NULL;
 
       /*
       ** Place table entries on free lists for use again.
@@ -1167,6 +1194,7 @@ main (int argc, char **argv)
 
       titlebuf[0] = '\0';
       deckname[0] = '\0';
+      DBG_MEMCHECK();
    }
 
    /*
@@ -1174,10 +1202,13 @@ main (int argc, char **argv)
    */
 
 
-   fclose (infd);
-   fclose (outfd);
-   fclose (tmpfd0);
-   if (listmode)
+   if (infd)
+      fclose (infd);
+   if (outfd)
+      fclose (outfd);
+   if (tmpfd0)
+      fclose (tmpfd0);
+   if (listmode && lstfd)
       fclose (lstfd);
 
    /*
@@ -1190,5 +1221,9 @@ main (int argc, char **argv)
    if (status != 0)
       unlink (outfile);
 
+#ifdef DEBUGMALLOCSDUMP
+   DBG_MEMDUMP();
+#endif
+
    return (status == 0 ? NORMAL : ABORT);
 }
